#!/bin/sh
#
# Nagios plugin to check number of processes on Solaris (using regexp).
#
# Should be replaced by a version of check_procs that supports regular
# expressions (if such plugin becomes available in Nagios Plugins) and
# that can be executed by any user (not just root).
#
# $Id: check_solaris_procs_regexp.sh,v 1.3 2008/09/22 12:33:24 kivimaki Exp $
#
# Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

# ------------------------------ SETTINGS --------------------------------------

# External commands
CMD_AWK="/usr/bin/awk"
CMD_EGREP="/usr/bin/egrep"
CMD_PS="/usr/bin/ps"

# Nagios plugin return values
STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_TEXT=""

# Default values
PROC_USER=""
PROC_USER_U=""
PROC_COUNT=0
PROC_WARN_MIN=-1
PROC_WARN_MAX=999999
PROC_CRIT_MIN=-1
PROC_CRIT_MAX=999999
REGEXP=""
WARN_TRIGGER=0
CRIT_TRIGGER=0
VERBOSE=0

# ------------------------------ FUNCTIONS -------------------------------------

printInfo() {
    echo "Nagios plugin to check number of processes (using regular expressions)."
    echo "Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd."
}

printHelp() {
    echo
    echo "Usage: `basename $0` -w <range> -c <range> [ -u <user> ] -e <regexp>"
    echo
    echo "  -w  warning safe range ([min]:[max])"
    echo "  -c  critical safe range ([min]:[max])"
    echo "  -u  only count processes owned by this user"
    echo "  -e  regular expression (for $CMD_EGREP)"
    echo "  -h  this help screen"
    echo "  -l  license info"
    echo "  -v  verbose output (for debugging)"
    echo "  -V  version info"
    echo
    echo "Note about safe ranges: You can give the minimum (min:) or maximum (:max)"
    echo "number of processes or both (min:max). Values must be positive integers"
    echo "and mins must be less than maxes."
    echo
    echo "Example: `basename $0` -w :200 -c 1: -u oracle -e 'ora_(pmon|smon)_(.*)'"
    echo "This will return CRITICAL if there are no processes matching the regexp,"
    echo "WARNING if there are more than 200 and OK otherwise"
    echo
}

printLicense() {
    echo
    echo "This program is free software; you can redistribute it and/or"
    echo "modify it under the terms of the GNU General Public License"
    echo "as published by the Free Software Foundation; either version 2"
    echo "of the License, or (at your option) any later version."
    echo
    echo "This program is distributed in the hope that it will be useful,"
    echo "but WITHOUT ANY WARRANTY; without even the implied warranty of"
    echo "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
    echo "GNU General Public License for more details."
    echo
    echo "You should have received a copy of the GNU General Public License"
    echo "along with this program; if not, write to the Free Software"
    echo "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."
    echo
}

printVersion() {
    echo
    echo "\$Id: check_solaris_procs_regexp.sh,v 1.3 2008/09/22 12:33:24 kivimaki Exp $"
    echo
}

# Checks command line options (pass $@ as parameter).
checkOptions() {
    if [ $# -eq 0 ]; then
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    fi

    while getopts w:c:u:e:lhvV OPT "$@"; do
            case $OPT in
                w) # warning range
                   opt_warn_min=`echo $OPTARG | $CMD_AWK 'BEGIN{FS=":"}{print \$1}'`
                   opt_warn_max=`echo $OPTARG | $CMD_AWK 'BEGIN{FS=":"}{print \$2}'`
                   WARN_TRIGGER=1
                   ;;
                c) # critical range
                   opt_crit_min=`echo $OPTARG | $CMD_AWK 'BEGIN{FS=":"}{print \$1}'`
                   opt_crit_max=`echo $OPTARG | $CMD_AWK 'BEGIN{FS=":"}{print \$2}'`
                   CRIT_TRIGGER=1
                   ;;
                u) # user
                   PROC_USER="$OPTARG"
                   PROC_USER_U="-u $OPTARG"
                   ;;
                e) # regexp
                   REGEXP="$OPTARG"
                   ;;
                l) printInfo
                   printLicense
                   exit $STATE_UNKNOWN
                   ;;
                h) printInfo
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
                v) VERBOSE=1
                   ;;
                V) printInfo
                   printVersion
                   exit $STATE_UNKNOWN
                   ;;
                ?) printInfo
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
    done

    if [ "$REGEXP" = "" ]; then
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    fi

    range_error=0
    if [ $WARN_TRIGGER -eq 1 ]; then
        if [ "$opt_warn_min" != "" ]; then
            if [ "`echo $opt_warn_min | grep '^[0-9]*\$'`" = "" ]; then
                range_error=1
            fi
            PROC_WARN_MIN=$opt_warn_min
        fi
        if [ "$opt_warn_max" != "" ]; then
            if [ "`echo $opt_warn_max | grep '^[0-9]*\$'`" = "" ]; then
                range_error=1
            fi
            PROC_WARN_MAX=$opt_warn_max
        fi
        if [ $PROC_WARN_MIN -ge $PROC_WARN_MAX ]; then
            range_error=1
        fi
    fi
    if [ $CRIT_TRIGGER -eq 1 ]; then
        if [ "$opt_crit_min" != "" ]; then
            if [ "`echo $opt_crit_min | grep '^[0-9]*\$'`" = "" ]; then
                range_error=1
            fi
            PROC_CRIT_MIN=$opt_crit_min
        fi
        if [ "$opt_crit_max" != "" ]; then
            if [ "`echo $opt_crit_max | grep '^[0-9]*\$'`" = "" ]; then
                range_error=1
            fi
            PROC_CRIT_MAX=$opt_crit_max
        fi
        if [ $PROC_CRIT_MIN -ge $PROC_CRIT_MAX ]; then
            range_error=1
        fi
    fi
    if [ $range_error -eq 1 ]; then
        echo "Error: Invalid safe range values."
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    fi

    if [ -z "$PROC_USER_U" ]; then
        # ask ps to print all processes as user was not defined
        PROC_USER_U="-e"
    fi
}

# ----------------------------- MAIN PROGRAM -----------------------------------

checkOptions "$@"

if [ $VERBOSE -eq 1 ]; then
    echo "Executing $CMD_PS $PROC_USER_U -o user,group,comm,args | $CMD_EGREP -c -e '$REGEXP'..."
fi
PROC_COUNT=`$CMD_PS $PROC_USER_U -o user,group,comm,args | $CMD_EGREP -c -e "$REGEXP"`

if [ "$PROC_USER" != "" ]; then
    STATE_TEXT="- $PROC_COUNT processes, user $PROC_USER, regexp '$REGEXP'"
else
    STATE_TEXT="- $PROC_COUNT processes, regexp '$REGEXP'"
fi

if [ $CRIT_TRIGGER -eq 1 ]; then
    if [ $PROC_COUNT -lt $PROC_CRIT_MIN ] || [ $PROC_COUNT -gt $PROC_CRIT_MAX ]; then
        echo "PROCS CRITICAL $STATE_TEXT|procs=$PROC_COUNT"
        exit $STATE_CRITICAL
    fi
fi
if [ $WARN_TRIGGER -eq 1 ]; then
    if [ $PROC_COUNT -lt $PROC_WARN_MIN ] || [ $PROC_COUNT -gt $PROC_WARN_MAX ]; then
        echo "PROCS WARNING $STATE_TEXT|procs=$PROC_COUNT"
        exit $STATE_WARNING
    fi
fi

echo "PROCS OK $STATE_TEXT|procs=$PROC_COUNT"
exit $STATE_OK
